# пример использования
# image test = Par("bg", "layer1", "layer2")
# или
# show expression Par("bg", "layer1", "layer2") as room

# картинки должны быть разного размера, например,
# 3130х1760, 2614х1470, 2099х1180

# либо одинакового размера, но тогда их нужно увеличивать программно:
# Par(("bg", 1.5), ("layer1", 1.3), ("layer2",1.15))

init -1111 python:
    # на случай, если не используется модуль для пузырей с диалогами
    speaker = None

    # положение говорящего, если он часть параллакса
    parallax_tag = None
    parallax_x, parallax_y, parallax_w, parallax_h = -1, -1, -1, -1

init python:
    import math

    # класс для параллакса
    # Extend the Par class to allow adding objects dynamically
    class Par(renpy.Displayable):
        def __init__(self, *args):
            super(Par, self).__init__()
            self.mouse_x, self.mouse_y = int(config.screen_width / 2), int(config.screen_height / 2)
            self.actual_x = None
            self.actual_y = None
            self.last_st = 0

            # Initialize layers
            self.images = []
            for i in args:
                image, zoom = i, 1
                if isinstance(i, (tuple, list)):
                    image, zoom = i[0], i[1]
                self.images.append((str(image), Transform(image, zoom=zoom)))

        def add_layer(self, image, zoom=1):
            """
            Adds a new layer to the parallax dynamically.
            :param image: The image file or displayable for the layer.
            :param zoom: The zoom level for the layer.
            """
            self.images.append((str(image), Transform(image, zoom=zoom)))
            renpy.redraw(self, 0)

        def render(self, width, height, st, at):
            render = renpy.Render(width, height)

            for sprite_name, image in self.images:
                cr = renpy.render(image, 0, 0, 0, 0)
                w, h = cr.get_size()

                if self.actual_x is None:
                    self.actual_x = width / 2
                    self.actual_y = height / 2

                minimum_speed = 0.5
                maximum_speed = 3
                speed = 1 + minimum_speed
                dx = min(maximum_speed, max(minimum_speed, self.mouse_x - self.actual_x))
                dy = min(maximum_speed, max(minimum_speed, self.mouse_y - self.actual_y))
                st_change = st - self.last_st
                self.last_st = st
                self.actual_x = math.floor(self.actual_x + (self.mouse_x - self.actual_x) * speed * st_change)
                self.actual_y = math.floor(self.actual_y + (self.mouse_y - self.actual_y) * speed * st_change)

                x = self.actual_x - self.actual_x * w / width
                y = self.actual_y - self.actual_y * h / height

                if store.speaker:
                    if str(store.speaker) in str(sprite_name):
                        store.parallax_tag = speaker
                        store.parallax_x = x
                        store.parallax_y = y
                        store.parallax_w = w
                        store.parallax_h = h

                r = renpy.render(image, int(w), int(h), st, at)
                render.blit(r, (int(x), int(y)))

            renpy.redraw(self, 0)
            return render

        def event(self, ev, x, y, st):
            import pygame
            if ev.type == pygame.MOUSEMOTION:
                if (x != self.mouse_x) or (y != self.mouse_y):
                    self.mouse_x, self.mouse_y = x, y
            return





# Battle Character Parallax


default persistent.parallax = True #Add a toggle in your settings! Your players will appreciate it.

transform parallax_arena_bg:
    zoom 1.2
    xalign 0.5
    ypos -50
    #perspective True
    #subpixel True
    function moving_camera

# Distances (define more transforms as needed).

transform near:
    truecenter()
    ypos int(config.screen_height*0.528) #shifting the image downwards a bit so the parallax effect doesnt show its cutoff

transform far:
    truecenter()
    xpos 400
    zzoom True
    zpos -750

# Setting our images to be near by default, and bgs as far.

define config.default_transform = near
define config.tag_transform = {"bg" : far}

# Moving camera logic

init python:
    def moving_camera(trans, st, at):
        if persistent.parallax:
            x, y = renpy.display.draw.get_mouse_pos()
            
            parallax_factor = 0.055
            target_xoffset = (x - config.screen_width / 2) * parallax_factor
            target_yoffset = (y - config.screen_height / 2) * parallax_factor
            
            easing_factor = 0.055 #closer to 0 = slower, closer to 1 = faster, with 1 being no ease
            trans.xoffset += (target_xoffset - trans.xoffset) * easing_factor
            trans.yoffset += (target_yoffset - trans.yoffset) * easing_factor
        else:
            trans.xoffset = 0
            trans.yoffset = 0
        
        return 0


transform parallax_arena:
    zoom 0.9
    ypos -300
    xalign 0.5
    #perspective True
    #subpixel True
    function moving_camera3

init python:
    def moving_camera3(trans, st, at):
        if persistent.parallax:
            x, y = renpy.display.draw.get_mouse_pos()
            
            parallax_factor = 0.055
            target_xoffset = (x - config.screen_width / 2) * parallax_factor
            target_yoffset = (y - config.screen_height / 2) * parallax_factor
            
            easing_factor = 0.055 #closer to 0 = slower, closer to 1 = faster, with 1 being no ease
            trans.xoffset += (target_xoffset - trans.xoffset) * easing_factor
            trans.yoffset += (target_yoffset - trans.yoffset) * easing_factor
        else:
            trans.xoffset = 0
            trans.yoffset = 0
        
        return 0




# Enemy Character parallax_enemy

transform parallax_enemy:
    zoom 0.5
    align((0.8, 0.6))
    #perspective True
    #subpixel True
    function moving_camera2
transform parallax_enemy2:
    zoom 0.5
    align((0.55, 0.6))
    #perspective True
    #subpixel True
    function moving_camera2

# Distances (define more transforms as needed).
# Moving camera logic

init python:
    def moving_camera2(trans, st, at):
        if persistent.parallax:
            x, y = renpy.display.draw.get_mouse_pos()
            
            parallax_factor = 0.02
            target_xoffset = (x - config.screen_width / 2) * parallax_factor
            target_yoffset = (y - config.screen_height / 2) * parallax_factor
            
            easing_factor = 0.055 #closer to 0 = slower, closer to 1 = faster, with 1 being no ease
            trans.xoffset += (target_xoffset - trans.xoffset) * easing_factor
            trans.yoffset += (target_yoffset - trans.yoffset) * easing_factor
        else:
            trans.xoffset = 0
            trans.yoffset = 0
        
        return 0



    # Enemy Character Parallax



transform parallax_character:
    align((0.0, 0.5))
    zoom 0.5
    #perspective True
    #subpixel True
    function moving_camera3

init python:
    def moving_camera3(trans, st, at):
        if persistent.parallax:
            x, y = renpy.display.draw.get_mouse_pos()
            
            parallax_factor = 0.04
            target_xoffset = (x - config.screen_width / 2) * parallax_factor
            target_yoffset = (y - config.screen_height / 2) * parallax_factor
            
            easing_factor = 0.055 #closer to 0 = slower, closer to 1 = faster, with 1 being no ease
            trans.xoffset += (target_xoffset - trans.xoffset) * easing_factor
            trans.yoffset += (target_yoffset - trans.yoffset) * easing_factor
        else:
            trans.xoffset = 0
            trans.yoffset = 0
        
        return 0